/*
 * Copyright © 2021 Fachhochschule für die Wirtschaft (FHDW) Hannover
 *
 * This file is part of ipspiel21-Dilemma.
 *
 * Ipspiel21-Dilemma is free software: you can redistribute it and/or modify it under the terms of the GNU
 * General Public License as published by the Free Software Foundation, either version 3 of the License,
 * or (at your option) any later version.
 *
 * Ipspiel21-Dilemma is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with ipspiel21-Dilemma. If not, see
 * <http://www.gnu.org/licenses/>.
 */
package de.fhdw.gaming.ipspiel21.dilemmaOriginal.domain;

import de.fhdw.gaming.core.domain.GameBuilder;
import de.fhdw.gaming.core.domain.GameException;
import de.fhdw.gaming.core.domain.ObserverFactoryProvider;

/**
 * A builder which allows to create a Dilemma game.
 */
public interface DilemmaGameBuilder extends GameBuilder {

    /**
     * Creates an {@link DilemmaPlayerBuilder} which allows to create and add a player to the game together with her
     * strategy.
     */
    DilemmaPlayerBuilder createPlayerBuilder();

    /**
     * Adds a player builder and the corresponding strategy.
     *
     * @param playerBuilder The builder player used for creating the player.
     * @param strategy      The player's strategy.
     * @throws GameException if adding the player is not allowed by the rules of the game.
     */
    DilemmaGameBuilder addPlayerBuilder(DilemmaPlayerBuilder playerBuilder, DilemmaStrategy strategy)
            throws GameException;

    /**
     * Changes the {@link ObserverFactoryProvider}.
     *
     * @param newObserverFactoryProvider The new {@link ObserverFactoryProvider}.
     * @return {@code this}
     */
    DilemmaGameBuilder changeObserverFactoryProvider(ObserverFactoryProvider newObserverFactoryProvider);

    @Override
    DilemmaGame build(int id) throws GameException, InterruptedException;
}
