/*
 * Copyright © 2021 Fachhochschule für die Wirtschaft (FHDW) Hannover
 *
 * This file is part of ipspiel21-Dilemma.
 *
 * Ipspiel21-Dilemma is free software: you can redistribute it and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software Foundation, either version 3 of the License, or 
 * (at your option) any later version.
 *
 * Ipspiel21-Dilemma is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with ipspiel21-Dilemma. If not, see
 * <http://www.gnu.org/licenses/>.
 */
package de.fhdw.gaming.ipspiel21.dilemmaOriginal.domain;

import java.util.Map;

import de.fhdw.gaming.core.domain.GameException;
import de.fhdw.gaming.ipspiel21.dilemmaOriginal.moves.impl.AbstractDilemmaMove;

/**
 * A builder which allows to create a Dilemma player.
 */
public interface DilemmaPlayerBuilder {

    /**
     * Changes the name of the player.
     * <p>
     * There is no default.
     *
     * @param newName The name of the player.
     * @return {@code this}
     */
    DilemmaPlayerBuilder changeName(String newName);

    /**
     * Changes the possible outcomes of the player.
     * <p>
     * There is no default.
     *
     * @param possibleOutcomes The possible outcomes of the player. The key for the first-level map is the answer of the
     *                         first player, the key for the second-level map is the answer of the second player.
     */
    DilemmaPlayerBuilder changePossibleOutcomes(
            Map<AbstractDilemmaMove, Map<AbstractDilemmaMove, Double>> possibleOutcomes);

    /**
     * Builds the player.
     *
     * @param state The Dilemma game state.
     * @return The Dilemma player.
     * @throws GameException if creating the player is not allowed by the rules of the game.
     */
    DilemmaPlayer build(DilemmaState state) throws GameException;

    /**
     * Builds a default player builder.
     *
     *@return The Dilemma player builder.
     */
    DilemmaPlayerBuilder defaultPlayerBuilder();

}
